<?php
  require_once("/usr/share/mercaware/webstart/clients.php");

  $versions = array();

  // Walk through all configurations for the PostgreSQL database users
  foreach($config as $name => $client)
  {
    if(array_key_exists("home", $client))
    {
      $home = escapeshellcmd($client['home']);
      $cmds = array("cd " . $home,
                    "if [ -f .bashrc ]; then . " . $home . "/.bashrc >> /dev/null 2>&1; fi",
                    "if [ -f .profile ]; then . " . $home . "/.profile >> /dev/null 2>&1; fi",
                    "MXROOT=" . $home . " S2RROOT=\$MXROOT . " . $home . "/etc/mxdbsrc >> /dev/null 2>&1",
                    "echo -n \$PGHOST#\$PGPORT#\$PGDATABASE#\${PGUSER:-\$MXDBUSER}#\${PGPASSWORD:-\$MXDBPASSWD}#",
                    $home . "/src/programs/mxd -V 2>&1");

      // Unfortunately not very elegant, but working
      list($server, $port, $database, $username, $password, $mxd) = explode("#", trim(shell_exec(implode("; ", $cmds))));

      // Hackish parsing of mercaware version data
      if(preg_match("/^\d+\.\d+\.\d+\.\d+$/", ($mxd_version = preg_replace("/.*release ([^,]+), .*/", "$1", $mxd))))
      {
        $versions[$name]['mercaware-Applikationsserver']['version'] = preg_replace("/.*release ([^,]+), .*/", "$1", $mxd);

        if(($mxd_tstamp = strtotime(preg_replace("/.*, (.*)$/", "$1", $mxd))) > 0)
        {
          $versions[$name]['mercaware-Applikationsserver']['tooltip'] = "Release vom " . date("d.m.Y", $mxd_tstamp);
        }
      }
      else
      {
        $versions[$name]['mercaware-Applikationsserver']['version'] = "Unbekannt";
        $versions[$name]['mercaware-Applikationsserver']['tooltip'] = "Fehler beim Auslesen der Version";
      }

      // Parsing Subversion data as elegant as possible
      if(is_dir($home . "/src/.svn/") && is_dir($home . "/.subversion/"))
      {
        if(function_exists("libxml_use_internal_errors"))
        {
          libxml_use_internal_errors(true);
        }

        $xml = simplexml_load_string(trim(shell_exec("LANG=C svn info --xml " . $home . "/src/ --config-dir " . $home . "/.subversion/ 2> /dev/null")));

        if($xml)
        {
          if(strpos($xml->entry->url, "/trunk") !== false)
          {
            $versions[$name]['mercaware-Businesslogik']['version'] = "Trunk, Rev. " . $xml->entry->commit['revision'];
            $versions[$name]['mercaware-Businesslogik']['tooltip'] = "Entwicklungszweig vom " . date("d.m.Y H:i:s O", strtotime($xml->entry->commit->date));
          }
          elseif(strpos($xml->entry->url, "/branches/") !== false)
          {
            $versions[$name]['mercaware-Businesslogik']['version'] = preg_replace("/.*\/branches\/(.*)/", "$1", $xml->entry->url) . ", Rev. " . $xml->entry->commit['revision'];
            $versions[$name]['mercaware-Businesslogik']['tooltip'] = "Branch vom " . date("d.m.Y H:i:s O", strtotime($xml->entry->commit->date));
          }
          elseif(strpos($xml->entry->url, "/tags/") !== false)
          {
            $versions[$name]['mercaware-Businesslogik']['version'] = preg_replace("/.*\/tags\/(.*)/", "$1", $xml->entry->url);
            $versions[$name]['mercaware-Businesslogik']['tooltip'] = "Release vom " . date("d.m.Y H:i:s O", strtotime($xml->entry->commit->date));
          }
          else
          {
            $versions[$name]['mercaware-Businesslogik']['version'] = "Unbekannt";
            $versions[$name]['mercaware-Businesslogik']['tooltip'] = "Unbekannter Subversion-Zweig: " . $xml->entry->url;
          }
        }
        else
        {
          $versions[$name]['mercaware-Businesslogik']['version'] = "Unbekannt";
          $versions[$name]['mercaware-Businesslogik']['tooltip'] = "Fehler beim Auslesen der Subversion-Daten";
        }
      }
      else
      {
        $rpm = str_replace("mercaware(businesslogic) = ", "", trim(shell_exec("rpm -q --provides raspberrypi-mercaware 2>&1 | grep -m 1 'mercaware(businesslogic) = '")));

        if($rpm)
        {
          $versions[$name]['mercaware-Businesslogik']['version'] = $rpm;
          $versions[$name]['mercaware-Businesslogik']['tooltip'] = "Release für erp case (" . php_uname("m") . ")"; 
        }
        else
        {
          $versions[$name]['mercaware-Businesslogik']['version'] = "Unbekannt";
          $versions[$name]['mercaware-Businesslogik']['tooltip'] = "Businesslogik verwendet kein Subversion";
        }
      }

      // Pentaprise favoured using $HOME/.odbc.ini
      $odbc = @parse_ini_file($client['home'] . "/.odbc.ini", true);

      $config['db'][$name]['server'] = (!empty($server) ? $server : (!isset($odbc['PostgreSQL']['Servername']) ? $odbc['PostgreSQL']['Servername'] : "localhost"));
      $config['db'][$name]['port'] = (!empty($port) ? $port : (!isset($odbc['PostgreSQL']['Port']) ? $odbc['PostgreSQL']['Port'] : 5432));
      $config['db'][$name]['database'] = (!empty($database) ? $database : (!isset($odbc['PostgreSQL']['Database']) ? $odbc['PostgreSQL']['Database'] : "MXDBS"));
      $config['db'][$name]['username'] = (!empty($username) ? $username : "postgres");
      $config['db'][$name]['password'] = (!empty($password) ? $password : "");

      // Connect PostgreSQL database and fetch server version information
      $link = @pg_connect("host='" . $config['db'][$name]['server'] . "' " .
                          "port='" . $config['db'][$name]['port'] . "' " .
                          "dbname='" . $config['db'][$name]['database'] . "' " .
                          "user='" . $config['db'][$name]['username'] . "' " .
                          "password='" . $config['db'][$name]['password'] . "'");

      if($link)
      {
        $pg_versions = pg_version($link);
        $versions[$name]['PostgreSQL-Datenbankserver']['version'] = $pg_versions['server'];

        @pg_close($link);
      }
      else
      {
        $versions[$name]['PostgreSQL-Datenbankserver']['version'] = "Unbekannt";
        $versions[$name]['PostgreSQL-Datenbankserver']['tooltip'] = "Fehler beim Verbinden zur Datenbank";
      }
    }
  }

  // Walk through all elements in configuration
  foreach(array_keys($config) as $client)
  {
    // Only proceed if element is a mercaware client
    if(!in_array($client, array("global", "visible", "db")))
    {
      // Ignore client invisibility if there's a visibility list for this host
      if(!isset($config['visible'][$_SERVER['SERVER_NAME']]) && !isset($config['visible'][$_SERVER['SERVER_ADDR']]))
      {
        // Set invisible if client option is exists and is set
        if(isset($config[$client]['invisible']) && $config[$client]['invisible'])
        {
          unset($config[$client]);
        }
      }

      // Walk through all visibility hostnames
      if(isset($config['visible']))
      {
        foreach(array_keys($config['visible']) as $host)
        {
          // If value is not an array, make it an array
          if(!is_array($config['visible'][$host]))
          {
            $config['visible'][$host] = array($config['visible'][$host]);
          }

          // Set invisible if hostname matches and client is not on visible list
          if(($host == $_SERVER['SERVER_NAME'] || $host == $_SERVER['SERVER_ADDR']) && !in_array($client, $config['visible'][$host]) && !in_array("*", $config['visible'][$host]))
          {
            unset($config[$client]);
          }

          // Set invisible if hostname does not match (avoid invalid subdomains)
          elseif(!isset($config['visible'][$_SERVER['SERVER_NAME']]) && !isset($config['visible'][$_SERVER['SERVER_ADDR']]))
          {
            unset($config[$client]);
          }
        }
      }
    }
  }

  // Remove hidden/invisible client versions from the version list
  foreach($versions as $client => $types)
  {
    if(!array_key_exists($client, $config))
    {
      unset($versions[$client]);
    }
  }

  // Necessary workaround, see: http://meiert.com/de/publications/articles/20041004/
  if(stristr(isset($_SERVER['HTTP_ACCEPT']) ? $_SERVER['HTTP_ACCEPT'] : "", "application/xhtml+xml") || stristr(isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : "", "W3C_Validator"))
  {
    header("Content-Type: application/xhtml+xml; charset=utf-8");
  }
  else
  {
    header("Content-Type: text/html; charset=utf-8");
  }

  echo "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">

<head>
<title>Versionsübersicht – mercaware</title>
<link rel="shortcut icon" type="image/x-icon" href="<?php echo htmlspecialchars($config['global']['baseurl']); ?>/favicon.ico" />
<link rel="stylesheet" type="text/css" href="<?php echo htmlspecialchars($config['global']['baseurl']); ?>/screen.css" />
<meta http-equiv="content-type" content="text/html; charset=utf-8" />
<meta http-equiv="pragma" content="no-cache" />
<!--[if lte IE 6]>
<script src="<?php echo htmlspecialchars($config['global']['baseurl']); ?>/pngfix.js"></script>
<script>
  DD_belatedPNG.fix('.active a, .link');
</script>
<style>
body{text-align:center;}
#container{width:800px;background:#fff;}
#left ul{margin-top:-23px;}
</style>
<![endif]--> 
</head>

<body>

<div id="wrapper">
  <h1>mercaware</h1>
  <div id="container">
    <h2>Versionsübersicht</h2>
<?php
  if($number = count($versions))
  {
?>
    <table>
      <thead>
        <tr>
          <th class="col_first">Mandant</th>
          <th>Name des Mandants</th>
          <th>Art der Version</th>
          <th class="col_last">Version</th>
        </tr>
      </thead>
      <tbody>
<?php
    $i = 0;

    foreach($versions as $client => $types)
    {
      foreach($types as $typ => $version)
      {
?>
        <tr class="row_<?php echo $i; ?><?php if($i == 0) { ?> row_first<?php } elseif($i == $number-1) { ?> row_last<?php } ?><?php echo ($i % 2) ? ' even' : ' odd'; ?>">
          <td class="col_first"><a title="Name: <?php echo htmlspecialchars($config[$client]['name']); ?>"><?php echo htmlspecialchars($client); ?></a></td>
          <td><?php echo htmlspecialchars($config[$client]['name']); ?></td>
          <td><?php echo htmlspecialchars($typ); ?></td>
          <td class="col_last"><?php if(isset($version['tooltip'])) { ?><a title="<?php echo htmlspecialchars($version['tooltip']); ?>"><?php } ?><?php echo htmlspecialchars($version['version']); ?><?php if(isset($version['tooltip'])) { ?></a><?php } ?></td>
        </tr>
<?php
        $i++;
      }
    }
?>
      </tbody>
    </table>
<?php
  }
  else
  {
?>
    <p style="text-align:center">Im Moment sind keine Mandanten vorhanden</p>
<?php
  }
?>
    <div class="clear"></div>
  </div>
</div>

</body>
</html>
