" Vim syntax file
" Filenames:	*.def, *.deh
" Language:	Mask definitions and formulars
" Maintainer:	Robert Scheck <robert.scheck@mercaware.de>
" Last Change:	2015 Feb 01
" Original:	Steffen Schuemann <s.schuemann@pobox.com>

" For version 5.x: Clear all syntax items
" For version 6.x: Quit when a syntax file was already loaded
if version < 600
  syntax clear
elseif exists("b:current_syntax")
  finish
endif

" Read the C syntax for C like code parts
if version < 600
  so <sfile>:p:h/c.vim
else
  runtime! syntax/c.vim
  unlet b:current_syntax
endif

syn keyword pTypes	OS_THUMBNAIL OS_COUNTRYSIGN OS_AREACODE OS_FAXNUMBER
syn keyword pTypes	OS_BARPD OS_DIAGPD OS_SUBMASK OS_CURFACT
syn keyword pTypes	OS_FORMPDBUTTON OS_FILENAME OS_DIRNAME OS_BOOL
syn keyword pTypes	OS_SBOOL OS_SINT OS_SSHORT OS_STEXT OS_SDATE OS_BUTTON
syn keyword pTypes	OS_DATE OS_ENUMPD OS_MASKPDBUTTON OS_ENUMRB
syn keyword pTypes	OS_CURRENCY OS_GIF OS_INT OS_SHORT OS_SDOUBLE
syn keyword pTypes	OS_DOUBLE OS_MAIL OS_TEXT OS_TEXTAREA
syn keyword pTypes	OS_TIME OS_UID OS_WWW OS_MASK OS_MODULE
syn keyword pTypes	OS_MENU OS_STATUS OS_LABEL OS_AIPD

syn keyword pMaskflags	OS_MFIELDLIST OS_NMGIVELIST OS_BEFORE OS_FIRST
syn keyword pMaskflags	OS_NMTAKELIST OS_SEPARATOR OS_INDEXLIST OS_SORTLIST
syn keyword pMaskflags	OS_NOMPD OS_POSMASK OS_AFTER OS_LAST

syn keyword pMenubar	OS_IMEXMENUBAR OS_MMENUBAR OS_SELECTMENUBAR
syn keyword pMenubar	OS_SURVOMENUBAR OS_SURVEYMENUBAR OS_ORDERMENUBAR

syn keyword pFlags	OS_TITLEPOS OS_TITLE OS_REGEXPR OS_POSITION
syn keyword pFlags	OS_PBELOW OS_PABOVE OS_PLEFT OS_PRIGHT
syn keyword pFlags	OS_DBINFO OS_NUMCOUNT OS_TITLELEN OS_SELF
syn keyword pFlags	OS_EMPTY OS_NEXTMASK OS_GIVEFIELDLIST OS_TAKEFIELDLIST
syn keyword pFlags	OS_REFERENCELIST OS_DEFAULT OS_SELDEFAULT OS_FORCE
syn keyword pFlags	OS_NOCHECK OS_VARCHARTEXT OS_COMBI OS_FORCEDEFAULT
syn keyword pFlags	OS_UNIQUEDEL OS_JOINFIELD OS_HEIGHT OS_REVERSE
syn keyword pFlags	OS_DENYNMPD OS_CURCOMBI OS_DEFLIST OS_MASKNAME
syn keyword pFlags	OS_ACCESS OS_VISLEN

syn keyword pFillflags	OS_FILLCOUNT OS_FILLDOCUDIR OS_FILLJAVATIME OS_FILLTELNO
syn keyword pFillflags	OS_FILLKREDITOR OS_FILLDEBITOR OS_FILLUID OS_FILLMASKNUM
syn keyword pFillflags	OS_SELUID OS_FILLTABNAME OS_FILLLANG OS_FILLACTDATE
syn keyword pFillflags	OS_FILLACTTIME OS_FILLACTCUR OS_FILLDEFCUR OS_FILLACTKW

syn keyword pAccess	OS_READ OS_WRITE OS_NOACCESS OS_EXEC

syn keyword pUpdate	OS_UPDATEMASK OS_UPDATEFIELD

syn keyword pFormscript	OS_UPDATEFORMSCRIPT OS_FORMPULLDOWN OS_PRINT
syn keyword pFormscript	OS_REPLACE OS_INSERT OS_UPDATE OS_DELETE
syn keyword pFormscript	OS_CREATEFORMSCRIPT OS_AFTER OS_BEFORE 
syn keyword pFormscript	FORMSCRIPT FORMPULLDOWN MASK INSERT
syn keyword pFormscript	UPDATE DELETE SCRIPT FORMULAR END
syn keyword pFormscript	SURVPRINT PRINT SEPARATOR

syn keyword pDeprecated	OS_ISSURVEY

syn region pIncluded	display contained start=+"+ skip=+\\\\\|\\"+ end=+"+
syn match  pIncluded	display contained "<[^>]*>"
syn match  pInclude	display "^\s*#\s*include\>\s*["<]" contains=pIncluded

syn match  pIdent	display "^\s*#\s*ident\>\s*["<]" contains=pIncluded

syntax region pComment	start="/\*" matchgroup=NONE end="\*/"

" Define the default highlighting
if !exists("did_c_syntax_inits")
  let did_c_syntax_inits = 1
  hi link pComment	Comment
  hi link pIncluded	Comment
  hi link pInclude	Special
  hi link pIdent	Special
  hi link pTypes	Type
  hi link pUpdate	Type
  hi link pMaskflags 	Conditional
  hi link pAccess	Conditional
  hi link pFlags	Conditional
  hi link pFillflags	String
  hi link pMenubar	Special
  hi link pFormscript	Number
  hi link pDeprecated	Comment
endif

let b:current_syntax = "mercaware"

" vim: ts=8
